@@ -1,7 +1,7 @@
-NUM_VERSION = (2012, 2, 1)
+NUM_VERSION = (2012, 3)
 VERSION = ".".join(str(nv) for nv in NUM_VERSION)
 
-
+from pudb.py3compat import raw_input
 
 
 CURRENT_DEBUGGER = []
@@ -46,46 +46,56 @@ def runscript(mainpyfile, args=None, pre_run="", steal_output=False):
             from subprocess import call
             retcode = call(pre_run, close_fds=True, shell=True)
             if retcode:
-                print "*** WARNING: pre-run process exited with code %d." % retcode
+                print("*** WARNING: pre-run process exited with code %d." % retcode)
                 raw_input("[Hit Enter]")
 
         status_msg = ""
 
         try:
             dbg._runscript(mainpyfile)
-        except SystemExit, se:
+        except SystemExit:
+            se = sys.exc_info()[1]
             status_msg = "The debuggee exited normally with status code %s.\n\n" % se.code
         except:
             dbg.post_mortem = True
             dbg.interaction(None, sys.exc_info())
 
-        def quit_debugger(w, size, key):
-            dbg.ui.quit_event_loop = ["quit"]
+        while True:
+            import urwid
+            pre_run_edit = urwid.Edit("", pre_run)
 
-        import urwid
-        pre_run_edit = urwid.Edit("", pre_run)
+            result = dbg.ui.call_with_ui(dbg.ui.dialog,
+                urwid.ListBox([urwid.Text(
+                    "Your PuDB session has ended.\n\n%s"
+                    "Would you like to quit PuDB or restart your program?\n"
+                    "You may hit 'q' to quit."
+                    % status_msg),
+                    urwid.Text("\n\nIf you decide to restart, this command will be run prior to "
+                    "actually restarting:"),
+                    urwid.AttrMap(pre_run_edit, "value")
+                    ]),
+                [
+                    ("Restart", "restart"),
+                    ("Examine", "examine"),
+                    ("Quit", "quit"),
+                    ],
+                focus_buttons=True,
+                bind_enter_esc=False,
+                title="Finished",
+                extra_bindings=[
+                    ("q", "quit"),
+                    ("esc", "examine"),
+                    ])
 
-        result = dbg.ui.call_with_ui(dbg.ui.dialog,
-            urwid.ListBox([urwid.Text(
-                "Your PuDB session has ended.\n\n%s"
-                "Would you like to quit PuDB or restart your program?\n"
-                "You may hit 'q' to quit."
-                % status_msg),
-                urwid.Text("\n\nIf you decide to restart, this command will be run prior to "
-                "actually restarting:"),
-                urwid.AttrMap(pre_run_edit, "value")
-                ]),
-            [
-                ("Restart", "restart"),
-                ("Quit", "quit"),
-                ],
-            focus_buttons=True,
-            bind_enter_esc=False,
-            title="Finished",
-            extra_bindings=[("q", quit_debugger)])
+            if result == "quit":
+                return
 
-        if result == "quit":
-            return
+            if result == "examine":
+                dbg.post_mortem = True
+                dbg.interaction(None, sys.exc_info(), show_exc_dialog=False)
+
+            if result == "restart":
+                break
 
         pre_run = pre_run_edit.get_edit_text()
 
@@ -149,4 +159,4 @@ def pm():
 
 
 if __name__ == "__main__":
-    print "You now need to type 'python -m pudb.run'. Sorry."
+    print("You now need to type 'python -m pudb.run'. Sorry.")
