@@ -4,14 +4,17 @@
 from __future__ import division
 import urwid
 import bdb
+import sys
 
 from pudb.settings import load_config, save_config
 CONFIG = load_config()
 save_config(CONFIG)
 
-
-
-
+from pudb.py3compat import PY3, raw_input
+if PY3:
+    _next = "__next__"
+else:
+    _next = "next"
 
 
 HELP_TEXT = """\
@@ -30,6 +33,7 @@ Keys:
     t - run to cursor
     e - show traceback [post-mortem or in exception state]
 
+    H - move to current line (bottom of stack)
     u - move up one stack frame
     d - move down one stack frame
 
@@ -124,7 +128,10 @@ class Debugger(bdb.Bdb):
         if steal_output:
             raise NotImplementedError("output stealing")
             import sys
-            from cStringIO import StringIO
+            if PY3:
+                from io import StringIO
+            else:
+                from cStringIO import StringIO
             self.stolen_output = sys.stderr = sys.stdout = StringIO()
             sys.stdin = StringIO("") # avoid spurious hangs
 
@@ -132,7 +139,7 @@ class Debugger(bdb.Bdb):
         from pudb.settings import save_breakpoints
         save_breakpoints([
             bp
-            for fn, bp_lst in self.get_all_breaks().iteritems()
+            for fn, bp_lst in self.get_all_breaks().items()
             for lineno in bp_lst
             for bp in self.get_breaks(fn, lineno)
             if not bp.temporary])
@@ -158,6 +165,9 @@ class Debugger(bdb.Bdb):
 
     def set_frame_index(self, index):
         self.curindex = index
+        if index < 0 or index >= len(self.stack):
+            return
+
         self.curframe, lineno = self.stack[index]
         self.ui.set_current_line(lineno, self.curframe.f_code.co_filename)
         self.ui.update_var_view()
@@ -183,13 +193,13 @@ class Debugger(bdb.Bdb):
 
         return stack, index
 
-    def interaction(self, frame, exc_tuple=None):
+    def interaction(self, frame, exc_tuple=None, show_exc_dialog=True):
         if exc_tuple is None:
             tb = None
         else:
             tb = exc_tuple[2]
 
-        if frame is None:
+        if frame is None and tb is not None:
             frame = tb.tb_frame
 
         found_bottom_frame = False
@@ -206,12 +216,14 @@ class Debugger(bdb.Bdb):
             return
 
         self.stack, index = self.get_shortened_stack(frame, tb)
+
         if self.post_mortem:
             index = len(self.stack)-1
 
         self.set_frame_index(index)
 
-        self.ui.call_with_ui(self.ui.interaction, exc_tuple)
+        self.ui.call_with_ui(self.ui.interaction, exc_tuple,
+                show_exc_dialog=show_exc_dialog)
 
     def get_stack_situation_id(self):
         return str(id(self.stack[self.curindex][0].f_code))
@@ -280,7 +292,10 @@ class Debugger(bdb.Bdb):
         # user_call for details).
         self._wait_for_mainpyfile = 1
         self.mainpyfile = self.canonic(filename)
-        statement = 'execfile( "%s")' % filename
+        if PY3:
+            statement = 'exec(compile(open("%s").read(), "%s", "exec"))' % (filename, filename)
+        else:
+            statement = 'execfile( "%s")' % filename
         self.run(statement, globals=globals_, locals=locals_)
 
 # }}}
@@ -291,8 +306,8 @@ class Debugger(bdb.Bdb):
 
 # UI stuff --------------------------------------------------------------------
 from pudb.ui_tools import make_hotkey_markup, labelled_value, \
-        SelectableText, SignalWrap, StackFrame, BreakpointFrame, \
-        SearchBox
+        SelectableText, SignalWrap, StackFrame, BreakpointFrame
+
 from pudb.var_view import FrameVarInfoKeeper
 
 
@@ -325,7 +340,9 @@ class DebuggerUI(FrameVarInfoKeeper):
         self.debugger = dbg
         Attr = urwid.AttrMap
 
-        self.search_box = None
+        from pudb.ui_tools import SearchController
+        self.search_controller = SearchController(self)
+
         self.last_module_filter = ""
 
         # {{{ build ui
@@ -552,12 +569,16 @@ class DebuggerUI(FrameVarInfoKeeper):
 
         self.stack_list.listen("enter", examine_frame)
 
+        def move_stack_top(w, size, key):
+            self.debugger.set_frame_index(len(self.debugger.stack)-1)
+
         def move_stack_up(w, size, key):
             self.debugger.move_up_frame()
 
         def move_stack_down(w, size, key):
             self.debugger.move_down_frame()
 
+        self.stack_list.listen("H", move_stack_top)
         self.stack_list.listen("u", move_stack_up)
         self.stack_list.listen("d", move_stack_down)
 
@@ -750,8 +771,6 @@ class DebuggerUI(FrameVarInfoKeeper):
         def page_up(w, size, key):
             w.keypress(size, "page up")
 
-        def move_up(w, size, key):
-            w.keypress(size, "up")
         def scroll_left(w, size, key):
             self.source_hscroll_start = max(
                     0,
@@ -765,35 +784,13 @@ class DebuggerUI(FrameVarInfoKeeper):
                 sl._invalidate()
 
         def search(w, size, key):
-            if self.search_box is None:
-                _, search_start = self.source.get_focus()
-
-                self.search_box = SearchBox(self)
-                self.search_AttrMap = urwid.AttrMap(
-                        self.search_box, "search box")
-
-                self.lhs_col.item_types.insert(
-                        0, ("flow", None))
-                self.lhs_col.widget_list.insert( 0, self.search_AttrMap)
-
-                self.columns.set_focus(self.lhs_col)
-                self.lhs_col.set_focus(self.search_AttrMap)
-            else:
-                self.columns.set_focus(self.lhs_col)
-                self.lhs_col.set_focus(self.search_AttrMap)
-                self.search_box.restart_search()
+            self.search_controller.open_search_ui()
 
         def search_next(w, size, key):
-            if self.search_box is not None:
-                self.search_box.do_search(1)
-            else:
-                self.message("No previous search term.")
+            self.search_controller.perform_search(dir=1, update_search_start=True)
 
         def search_previous(w, size, key):
-            if self.search_box is not None:
-                self.search_box.do_search(-1)
-            else:
-                self.message("No previous search term.")
+            self.search_controller.perform_search(dir=-1, update_search_start=True)
 
         def toggle_breakpoint(w, size, key):
             if self.shown_file:
@@ -834,7 +831,7 @@ class DebuggerUI(FrameVarInfoKeeper):
 
                 self.update_breakpoints()
             else:
-                raise RuntimeError, "no valid current file"
+                raise RuntimeError("no valid current file")
 
         def pick_module(w, size, key):
             from os.path import splitext
@@ -977,6 +974,7 @@ class DebuggerUI(FrameVarInfoKeeper):
         self.source_sigwrap.listen("b", toggle_breakpoint)
         self.source_sigwrap.listen("m", pick_module)
 
+        self.source_sigwrap.listen("H", move_stack_top)
         self.source_sigwrap.listen("u", move_stack_up)
         self.source_sigwrap.listen("d", move_stack_down)
 
@@ -1114,7 +1112,6 @@ class DebuggerUI(FrameVarInfoKeeper):
 
         self.top.listen("q", quit)
         self.top.listen("ctrl p", do_edit_config)
-        self.top.listen("H", help)
         self.top.listen("f1", help)
         self.top.listen("?", help)
 
@@ -1203,9 +1200,19 @@ class DebuggerUI(FrameVarInfoKeeper):
                 ("fixed", 1, urwid.SolidFill()),
                 w])
 
+        class ResultSetter:
+            def __init__(subself, res):
+                subself.res = res
+
+            def __call__(subself, w, size, key):
+                self.quit_event_loop = [subself.res]
+
         w = SignalWrap(w)
         for key, binding in extra_bindings:
-            w.listen(key, binding)
+            if isinstance(binding, str):
+                w.listen(key, ResultSetter(binding))
+            else:
+                w.listen(key, binding)
 
         w = urwid.LineBox(w)
 
@@ -1264,7 +1271,7 @@ class DebuggerUI(FrameVarInfoKeeper):
                 self.message("Package 'pygments' not found. "
                         "Syntax highlighting disabled.")
 
-        WELCOME_LEVEL = "e005"
+        WELCOME_LEVEL = "e007"
         if CONFIG["seen_welcome"] < WELCOME_LEVEL:
             CONFIG["seen_welcome"] = WELCOME_LEVEL
             from pudb import VERSION
@@ -1278,6 +1285,11 @@ class DebuggerUI(FrameVarInfoKeeper):
                     "look familiar.\n\n"
                     "If you're new here, welcome! The help screen (invoked by hitting "
                     "'?' after this message) should get you on your way.\n"
+                    "\nChanges in version 2012.3:\n\n"
+                    "- Python 3 support (contributed by Brad Froehle)\n"
+                    "- Better search box behavior (suggested by Ram Rachum)\n"
+                    "- Made it possible to go back and examine state from 'finished' window."
+                    " (suggested by Aaron Meurer)\n"
                     "\nChanges in version 2012.2.1:\n\n"
                     "- Don't touch config files during install.\n"
                     "\nChanges in version 2012.2:\n\n"
@@ -1289,15 +1301,15 @@ class DebuggerUI(FrameVarInfoKeeper):
                     "\nChanges in version 2011.3.1:\n\n"
                     "- Work-around for bug in urwid >= 1.0.\n"
                     "\nChanges in version 2011.3:\n\n"
-                    "- Finer-grained string highlighting (submitted by Aaron Meurer)\n"
-                    "- Prefs tweaks, instant-apply, top-down stack (submitted by Aaron Meurer)\n"
-                    "- Size changes in sidebar boxes (submitted by Aaron Meurer)\n"
-                    "- New theme 'midnight' (submitted by Aaron Meurer)\n"
-                    "- Support for IPython 0.11 (submitted by Chris Farrow)\n"
-                    "- Suport for custom stringifiers (submitted by Aaron Meurer)\n"
-                    "- Line wrapping in variables view (submitted by Aaron Meurer)\n"
+                    "- Finer-grained string highlighting (contributed by Aaron Meurer)\n"
+                    "- Prefs tweaks, instant-apply, top-down stack (contributed by Aaron Meurer)\n"
+                    "- Size changes in sidebar boxes (contributed by Aaron Meurer)\n"
+                    "- New theme 'midnight' (contributed by Aaron Meurer)\n"
+                    "- Support for IPython 0.11 (contributed by Chris Farrow)\n"
+                    "- Suport for custom stringifiers (contributed by Aaron Meurer)\n"
+                    "- Line wrapping in variables view (contributed by Aaron Meurer)\n"
                     "\nChanges in version 2011.2:\n\n"
-                    "- Fix for post-mortem debugging (submitted by 'Sundance')\n"
+                    "- Fix for post-mortem debugging (contributed by 'Sundance')\n"
                     "\nChanges in version 2011.1:\n\n"
                     "- Breakpoints saved between sessions\n"
                     "- A new 'dark vim' theme\n"
@@ -1311,7 +1323,6 @@ class DebuggerUI(FrameVarInfoKeeper):
             save_config(CONFIG)
             self.run_edit_config()
 
-
         try:
             if toplevel is None:
                 toplevel = self.top
@@ -1339,26 +1350,28 @@ class DebuggerUI(FrameVarInfoKeeper):
 
     # {{{ debugger-facing interface
 
-    def interaction(self, exc_tuple):
+    def interaction(self, exc_tuple, show_exc_dialog=True):
         self.current_exc_tuple = exc_tuple
 
         from pudb import VERSION
         caption = [(None,
-            u"PuDB %s - ?:help  n:next  s:step into  b:breakpoint  o:output  "
+            "PuDB %s - ?:help  n:next  s:step into  b:breakpoint  o:output  "
             "t:run to cursor  !:python shell"
             % VERSION)]
 
         if self.debugger.post_mortem:
             from traceback import format_exception
 
-            self.message(
-                    "The program has terminated abnormally because of an exception.\n\n"
-                    "A full traceback is below. You may recall this traceback at any "
-                    "time using the 'e' key. "
-                    "The debugger has entered post-mortem mode and will prevent further "
-                    "state changes.\n\n"
-                    + "".join(format_exception(*exc_tuple)),
-                    title="Program Terminated for Uncaught Exception")
+            if show_exc_dialog:
+                self.message(
+                        "The program has terminated abnormally because of an exception.\n\n"
+                        "A full traceback is below. You may recall this traceback at any "
+                        "time using the 'e' key. "
+                        "The debugger has entered post-mortem mode and will prevent further "
+                        "state changes.\n\n"
+                        + "".join(format_exception(*exc_tuple)),
+                        title="Program Terminated for Uncaught Exception")
+
             caption.extend([
                 (None, " "),
                 ("warning", "[POST-MORTEM MODE]")
@@ -1386,7 +1399,7 @@ class DebuggerUI(FrameVarInfoKeeper):
                     lines = getlines(fname)
 
                     from pudb.lowlevel import detect_encoding
-                    source_enc, _ = detect_encoding(iter(lines).next)
+                    source_enc, _ = detect_encoding(getattr(iter(lines), _next))
 
                     decoded_lines = []
                     for l in lines:
@@ -1399,7 +1412,6 @@ class DebuggerUI(FrameVarInfoKeeper):
                             decoded_lines, set(breakpoints))
                 except:
                     from traceback import format_exception
-                    import sys
 
                     self.message("Could not load source file '%s':\n\n%s" % (
                         fname, "".join(format_exception(*sys.exc_info()))),
@@ -1446,7 +1458,7 @@ class DebuggerUI(FrameVarInfoKeeper):
 
     def _get_bp_list(self):
         return [bp
-                for fn, bp_lst in self.debugger.get_all_breaks().iteritems()
+                for fn, bp_lst in self.debugger.get_all_breaks().items()
                 for lineno in bp_lst
                 for bp in self.debugger.get_breaks(fn, lineno)
                 if not bp.temporary]
