@@ -1,10 +1,17 @@
+from pudb.py3compat import PY3
+
 # breakpoint validity ---------------------------------------------------------
 def generate_executable_lines_for_code(code):
     l = code.co_firstlineno
     yield l
-    for c in code.co_lnotab[1::2]:
-        l += ord(c)
-        yield l
+    if PY3:
+        for c in code.co_lnotab[1::2]:
+            l += c
+            yield l
+    else:
+        for c in code.co_lnotab[1::2]:
+            l += ord(c)
+            yield l
 
 
 
@@ -80,6 +87,8 @@ def lookup_module(filename):
 import re
 cookie_re = re.compile("^\s*#.*coding[:=]\s*([-\w.]+)")
 from codecs import lookup, BOM_UTF8
+if PY3:
+    BOM_UTF8 = BOM_UTF8.decode()
 
 def detect_encoding(readline):
     """
@@ -108,7 +117,10 @@ def detect_encoding(readline):
 
     def find_cookie(line):
         try:
-            line_string = line.decode('ascii')
+            if PY3:
+                line_string = line
+            else:
+                line_string = line.decode('ascii')
         except UnicodeDecodeError:
             return None
 
